;+
; NAME:
;	F_Thick2_DEFAULTS
;
; PURPOSE: Function to return default values for
;   parameters, minimum and maximum range, and free parameter mask when
;   fitting to f_thick2 function.
;   
;   ***** To use a single power-law electron distribution, it is better to change the break energy to a value that is equal to
;           or larger than the high energy cutoff. Or one can set the low delta and the high delta to same value.
;
; CALLING SEQUENCE: defaults = f_thick2_defaults()
;
; INPUTS:
;	None
; OUTPUTS:
;	Structure containing default values
;
; MODIFICATION HISTORY:
; Kim Tolbert, February 2004
; 04/14/2009 Yang Su    Change the default values from [1,4, 600, 6, 10,32000] to 
;                                         [1.,  4., 150., 6., 20., 3.2e4]
;
;-
;------------------------------------------------------------------------------

function f_Thick2_defaults

defaults = { $
  fit_comp_params:      	 [1.,  4., 150., 6., 20., 3.2e4], $
  fit_comp_minima:           [1e-10,  1.1, 1., 1.1, 1., 100.], $
  fit_comp_maxima:           [1e10,  20,  1e5, 20,  1e3, 1e7], $
  fit_comp_free_mask:        [1,1,1,1,1,0] $
  }

return, defaults

end