;+
; PROJECT:
;	FERMI
; NAME:
;	FERMI_dn
;
; PURPOSE:
;	uses poshist file sc position to determine local day and night on FERMI
;
; CATEGORY:
;	QUICKLOOK AND CATALOG GENERATION FOR FERMI GBM
;
; CALLING SEQUENCE:
;
;
; CALLS:
;	none
;
; INPUTS:
;       Poshistfile - FERMI poshist (position history) daily file
;
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;       none explicit, only through commons;
;
; OPTIONAL OUTPUTS:
;	none
;
; KEYWORDS:
;	none
; COMMON BLOCKS:
;	none
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	none
;
; PROCEDURE:
;	none
;
; MODIFICATION HISTORY:
;	18-MAR-2010, richard.schwartz@nasa.gov
;
;-



function fermi_dn, poshistfile, ut=ut ;returns 1 for day, 0 for night for each time

default, hcrit, 7e4 ;critical altitude for x-rays

pos = gbm_rd_poshist(poshistfile, /utbase)
; mrdfits( poshistfile, 1, phdr)
;dateobs = anytim(fxpar(phdr,'date-obs'))
;deltaut =  anytim('1-jan-2001')-anytim('1-jan-1979')
utc = pos.utc
utdn = keyword_set(ut) ? anytim(ut) : utc

fld='POS_'+['X','Y','Z']
scxyz = fltarr(3, n_elements(utdn))
for i=0,2 do scxyz[i,0] = transpose(interpol(gt_tagval(pos,fld[i]),utc, utdn))

sun_occ, scxyz, utdn, hmin, smin

stest = -fix( f_div(smin,abs(smin)))>0
htest = fix(f_div(hmin, hcrit) <1 >0)

dnmask = byte(stest or htest)

if arg_present(ut) then result= dnmask else begin
	result = replicate({ut:anytim(utdn[0],/ints), mask:dnmask[0]},n_elements(dnmask))
	result.ut= anytim(utdn,/ints)
	result.mask = dnmask
	endelse
return, result
end
