;+
; Name: FERMI_MET2TIM
;
; Purpose: Convert Fermi Mission Elapsed time (seconds since 1-Jan-2001 with 
;   no leap sec)to seconds since 1-Jan-1979 with leap sec
; 
; Calling sequence:  tim = fermi_met2tim(met)
; 
; Argument:
;  met - mission elapsed time - should be double precision for full accuracy
; 
; Output - time in seconds since 1-Jan-1979
; 
; Written, Kim Tolbert 14-Mar-2011
; Modifications:
; 07-Nov-2011. Kim.  Modified header doc.
; 
;-

function fermi_met2tim, met

ref_utc = anytim('1-jan-2001 00:00:00.000')

time = met + ref_utc

nleap = fermi_nleap_sec(time)

time = time + nleap

return, time
end