;+
; PROJECT:
;	FERMI
; NAME:
;
;
; PURPOSE:
;	uses poshist quaternions to determine FERMI and nai orientation
;		to the Sun
;
; CATEGORY:
;	QUICKLOOK AND CATALOG GENERATION FOR FERMI GBM
;
; CALLING SEQUENCE:
;
;
; CALLS:
;	none
;
; INPUTS:
;       Poshistfile - Single daily poshist file for FERMI/GBM
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;       none explicit, only through commons;
;
; OPTIONAL OUTPUTS:
;	none
;
; KEYWORDS:
;	none
; COMMON BLOCKS:
;	none
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	none
;
; PROCEDURE:
;	none
;
; MODIFICATION HISTORY:
;	18-MAR-2010, richard.schwartz@nasa.gov
;
;-



function gbm_det_cosines, poshistfile, utc=utc, angle=angle
;Detector ID No.	Azimuth (deg)	Zenith (deg) ;FERMI coordinates
;0	45.9	20.6
;1	45.1	45.3
;2	58.4	90.2
;3	314.9	45.2
;4	303.2	90.3
;5	3.4		89.8
;6	224.9	20.4
;7	224.6	46.2
;8	236.6	90.0
;9	135.2	45.6
;10	123.7	90.4
;11	183.7	90.3
pf = gbm_get_filetype(poshistfile, 'poshist', count)
pos = gbm_rd_poshist(pf, /utbase) ; returns pos with utc for time in first field

detra=[45.9,45.1,58.4,314.9,303.2,3.4,224.9,$
	224.6,236.6,135.2,123.7,183.7]
detdec = 90.0-[20.6,45.3,90.2,45.2,90.3,89.8,20.4,$
	46.2,90.0,45.6,90.4,90.3]
utc = pos.utc
q = transpose([[pos.qsj_1], [pos.qsj_2], [pos.qsj_3], [pos.qsj_4]])
detxyz=sphcart(detra, detdec)
radec = solephut( utc )
sun = transpose(sphcart(reform( radec[0,*]), reform(radec[1,*])))
return, keyword_set(angle) ? acos(detxyz# qtvrot(sun, q,/inv))*!radeg : detxyz# qtvrot(sun, q,/inv)
end

