;+
; PROJECT:
;	FERMI
; NAME:
;	GBM_FIND_CONTIG
;
; PURPOSE:
;	This function returns the start and stop indices of
;	continuous Solar observing for the GBM nai detectors
;
; CATEGORY:
;	QUICKLOOK AND CATALOG GENERATION FOR FERMI GBM
;
; CALLING SEQUENCE:
;	onsun=gbm_find_contig(data, COUNT=COUNT)
;
; CALLS:
;	find_contig, anytim
;
; INPUTS:
;       Data - data structure from GBM_QLOOK
;
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;       none explicit, only through commons;
;
; OPTIONAL OUTPUTS:
;	COUNT=COUNT - number of intervals of continuous Solar observing for GBM
;
; KEYWORDS:
;	none
; COMMON BLOCKS:
;	none
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	none
;
; PROCEDURE:
;	Check the daynight flare and requires that the gap time between
;	GBM data rates is less than 1 minute. Normal SAA turnoffs will be longer
;
; MODIFICATION HISTORY:
;	18-MAR-2010, richard.schwartz@nasa.gov
;
;-



function gbm_find_contig, data, count = count, tmax=tmax

default, tmax, 60.0 ;maximum gap time to still be contiguous

utms = anytim(data.utm)
nel = n_elements(utms)
dutm = utms[1:*]-utms
test = where(dutm gt tmax, ntest)
dg = data.dn
dg[test>0] = 0
dg[test+1<(nel-1)] = 0
dg = where(dg)
contig = find_contig( dg, cg, cg2)
count  = n_elements(contig)
return, count eq 1 ? cg2 : transpose(dg[cg2])
end
