;+
; Name: gbm_match_rhessi
; 
; Purpose: Find rhessi flare that overlaps with GBM flare
; 
; Method: If rhessi start time is less than GBM flare end, or rhessi end time is greater than
;  GBM flare start, return the rhessi flare number.  If there's more than one flare that matches
;  that requirement, return the rhessi flare whose peak time is closest to the GBM peak time.
;  
; Calling sequence:  rflare = gbm_match_rhessi(gflare)
; 
; Calling Argument: flare - structure of GBM flare we want to find RHESSI flare for.
;   (structure is what we get from gbm_cat->getdata()
;   
; Written: Kim Tolbert
; Modifications:
;-

function gbm_match_rhessi, flare

rhessi_cat = hsi_read_flarelist(err_msg=err_msg)
if err_msg ne '' then message,'Error reading RHESSI Flare Catalog.'

nflare = n_elements(flare)
rhessi = lonarr(nflare)

for i=0,nflare-1 do begin
  q = where (rhessi_cat.start_time lt flare[i].utend and rhessi_cat.end_time gt flare[i].utstart, count)

  case count of
    0: 
    1: rhessi[i] = rhessi_cat[q].id_number
    else: begin
      diff = min( abs(rhessi_cat[q].peak_time - flare[i].utpeak), minind)
      rhessi[i] = rhessi_cat[q[minind]].id_number
      end
  endcase
endfor

return, rhessi

end