;+
; Name: gbm_time2posfile
;
; Purpose: Return approximate names of poshist file for given time range (so can be searched for)
; 
; Method: GBM poshist files are daily files named glg_poshist...yymmdd....fit.  For each date
;  in input time range construct that search string with '*' (wildcard) for characters we
;  don't care about.  Since times might be time range of cspec file (which starts a few seconds
;  on previous day, and ends a few seconds into next day), don't include days with less than
;  20 seconds in time range (otherwise when you're not online, even if you have poshist file
;  for day you want to plot, it will try to download file for previous day and fail).  
;  i.e. if time range is 10-Dec-2008 23:59:54.389 to 12-Dec-2008 00:00:01.128, just return
;  glg_poshist_*081211*.fit, instead of filenames for Dec 10, 11, and 12.
; 
; Argument:
;   time_range - scalar or range of times to return file names for
;
; Written: Kim Tolbert 27-Oct-2011
; Modifications:
;-
function gbm_time2posfile, time_range

times = anytim(time_range)
if n_elements(times) eq 1 then times = [times,times]

dates = timegrid( anytim(/date,times[0]), times[1],/days,/quiet)
if n_elements(dates) gt 1 and (times[0]-anytim(dates[0]) gt 86380.d0) then remove, 0, dates
last = n_elements(dates)-1
if n_elements(dates) gt 1 and (times[1]-anytim(dates[last]) lt 20.d0) then remove, last, dates 

yymmdd = time2file(dates, /date_only,/year2digit)

files = 'glg_poshist_*' + yymmdd + '*.fit'

return, files

end