;+
; Name: lat_plot_exposures
; 
; Purpose: Plot Lat Exposure data, with option to overplot calculated Good Time Intervals (GTI) and/or times
;   covered by LAT Response files
; 
; Calling Sequence:
;   lat_plot_exposures, date[, /show_gti, /show_rsp]
;    
; Input Argument:
;   date - time interval to plot exposure for (in anytim format). 2-element range, or if scalar assumes 1 day.
;   
; Input Keywords:
;   show_gti - If set, overplot the GTI intervals
;   show_rsp - If set, overplot the time intervals that the RSP files were made for
;  
; Output Keywords:
;   plotman_obj - returns plotman object reference 
;   
; Output: Plots the exposure information and GTI and RSP time info in a plotman instance.
;
;Method: Reads LAT daily Lightcurve (LC) file in hesperia LAT solar archive.  If user is not on hesperia computer,
;  copies file to user's local directory.  If requested time spans a day boundary concatenates data from each
;  daily file. GTI info also comes from LC file.  RSP times are found by parsing the file names of the RSP files
;  in the hesperia LAT RSP archive.
;  
; Examples:
; lat_plot_exposures, '25-feb-2014'
; lat_plot_exposures, ['25-feb-2014','27-feb-2014'], /show_gti, /show_rsp
;  
; Kim Tolbert, 6-Nov-2012
; Modifications: 
; 29-Feb-2016, Kim.  Extracted the innards of this to lat_plot_lc, and now call that with /expos flag.
;
;-

pro lat_plot_exposures, date, plotman_obj=pl, _extra=_extra

lat_plot_lc, /expos, date, plotman_obj=pl, _extra=_extra

end