;+
; Name: gbm2tim
;
; Purpose:  Convert GBM time in Modified Julian Day format to UTC (sec since 1-jan-1979 00:00 
; taking leap sec into account (so UTC will be smaller by # leap sec that have been inserted 
; between 1-jan-2001 and requested time).
;
; Calling sequence:  ut = gbm2tim(gbmt [, /relative])
;
; Arguments:
;  gbmt - GBM time in elapsed sec since 1-jan-2001 00:00 (no leap sec accounted for)
;  
; Keywords:
;  relative - if set, gbmt is relative to Fermi mission start time (1-jan-2001 00:00)
;
; Modifications:
; 15-Nov-2011, Kim.  Realized that this was no longer needed, so file renamed to old_gbm2tim
;   We thought fermi_trig_cat's times in mjd didn't include leap sec, but they do, so this isn't
;   needed.
;-

function gbm2tim, gbmt, relative=relative

ref_utc = anytim('1-jan-2001 00:00:00.000')

utc =anytim(mjd2any(gbmt))

if keyword_set(relative) then utc = utc + ref_utc

nleap = fermi_nleap_sec(utc)

return, utc + nleap
end
  