;+
;
; NAME:
;	list_sp_files
;
; PURPOSE:
;   Read spectrum or response matrix FITS file(s) and summarize contents.  Either pass in
;     file name(s), or it will operate on all '*spectrum*.fits' or '*srm*.fits' files in
;     current directory.
;
; CATEGORY:
;       SPECTRAL FITTING, SPEX - HESSI
;
; CALLING SEQUENCE: list_sp_files, files, rm=rm, out=out, nomore=nomore
;
; INPUTS:
;  FILES - name(s) of FITS file(s) to read. Optional.  If not provided, finds all of the
;    '*spectrum*.fits' or '*srm*.fits' files in the current directory.
;
; INPUT KEYWORDS:
;  RM - If set, summarize response matrix (rm) files.  Otherwise, spectrum files.
;  SRM - Same as RM.
;  NOMORE - If set, don't print output text on screen.
;
; OUTPUTS KEYWORDS:
;  OUT - Return string array containing output text.
;
; PROCEDURE: Create a temporary OSPEX data or drm object and call preview method for each file.
;
; Written 16-Jul-2004, Kim
;
; Modification History:
;	13-Aug-2004, Kim.  Added SRM keyword.
;-
;------------------------------------------------------------------------------
pro list_sp_files, files, rm=rm, srm=srm, out=out, nomore=nomore

checkvar, rm, srm, 0
checkvar, sp, (rm ne 1)

type = sp ? 'spectrum' : 'srm'

nfiles = n_elements(files)
if nfiles eq 0 then files = findfile('*'+type+'*.fits', count=nfiles)

if nfiles eq 0 then begin

	out='No ' + type + ' files found.'

endif else begin

	for i=0,nfiles-1 do begin
		if not exist(obj) then obj = sp ? obj_new('spex_data') : obj_new('spex_drm')
		if sp then obj -> set, spex_specfile=files[i] else obj -> set, spex_drmfile=files[i]
		obj->preview,out=this_out,/nomore
		out = append_arr(out, [' ', this_out[1:*]] )
	endfor

	obj_destroy,obj
endelse

if not keyword_set(nomore) then xmessage, out

end
