;+
;Name: messenger_mk_jpeg
;
;Purpose: Make daily jpegs plots showing messenger data and results of filtering for
; night, high temperature, voltage too low. Called from messenger_mk_movie or can call directly. Can
; only be run on hesperia (unless I modify it for location of input files).
; 
; messenger_mk_movie is run by softw on hesperia and files are written on hesperia in the messenger archive at
; /data/messenger/daily_plots.  An aaareadme.txt file in that location describes the
; plots.
; 
;Input Keywords:
; time - time to make jpegs for. If not provided, does whole mission. If scalar, then
;   does that one day.  If 2 elements, then it's the range of times.
; outdir - directory to write jpeg files in. Default is current directory.
;   
;Written: Kim Tolbert, 24-Jul-2014
;Modifications:
;  
;-

pro messenger_mk_jpeg, time=time_in, outdir=outdir

checkvar, outdir, curdir() + '/'
checkvar, time_in, ['1-aug-2004','1-jan-2050']
dates = anytim(time_in)
if n_elements(dates) eq 1 then dates = [dates, dates]

files = file_search('/data/messenger/', 'xrs*.dat', count=nfiles)

window,/free,xs=1200,ys=800
wid=!d.window
!p.thick=2
!p.charsize=2. 

for ii=0,nfiles-1 do begin
  fil = file_break(files[ii])
  year = strmid(fil, 3, 4)
  doy = strmid(fil, 7, 3)
  time = anytim(doy2utc(doy, year))
  if time ge dates[0] and time le dates[1] then begin
    jpeg_file = 'messenger_' + time2file(time,/date_only) + '.jpeg'
    read_messenger_pds,files=files[ii], data_str=d, /show_plot
    x2jpeg, outdir + jpeg_file, window=wid
  endif
endfor
  
end