;+
; Name: messenger_obs_times_write
;
; Purpose: Write text file containing time intervals when MESSENGER was observing the Sun.
;
; Input keywords:
;  time - scalar or 2-element range of times.  For each day in range, finds corresponding files on hesperia 
;    and calls messenger_obs_times to read files and return solar observing times.  Merges them into text file
;    /data/messenger/messenger_obs_times.txt.  Must be run on hesperia by softw.
;
; Output - text file /data/messenger/messenger_obs_times.txt containing start/end (VMS format) of solar obs times
;
; Written: Kim Tolbert, Nov. 2014
; Modifications:
;-


pro messenger_obs_times_write, time=time_in

outfile = '/data/messenger/messenger_obs_times.txt'

dates = anytim(time_in)
if n_elements(dates) eq 1 then dates = [dates, dates]
dates = anytim(dates,/vms)

m=obj_new('messenger')
files = m->search(dates[0], dates[1], count=count)
if count eq 0 then begin
  message,'No files found', /cont
  return
end

q = where (strpos(file_basename(files),'.dat') ne -1)
files = files[q]
files = str_replace(files, 'http://hesperia.gsfc.nasa.gov/', '/data/')

good = messenger_obs_times(files, count=count)

if count gt 0 then begin
  new = anytim(good, /vms)
  new = reform(new[0,*] + ', ' + new[1,*])
  if file_exist(outfile) then begin
    old = rd_tfile(outfile)
    oldsec = anytim(str2cols(old,','))
    stimes = [reform(oldsec[0,*]), reform(good[0,*])]
    s = sort(stimes)
    list = [old, new]
    list = list[s]
  endif else list = new
  prstr, list, file=outfile
endif

end




