;+
; Name: write_flare_background_text
;
; Purpose: Write a text file listing the files and times of the GRS flare data that can
;  be used for background-subtracted analysis.
;
; Written: Kim Tolbert 23-Apr-2019
; Modifications:
;-

pro write_flare_background_text

  restore, file='grs_dba_times.sav', /verbose
  d = out_dba
  usename = d.usename + 'FD'
  nd = n_elements(d)
  atd = '   ' + anytim(d.fd_use_time,/vms,/trunc)
  qb = where(d.db_use_time[0] ne 0. and d.da_use_time[0] eq 0., kb)
  qa = where(d.db_use_time[0] eq 0. and d.da_use_time[0] ne 0., ka)
  qboth = where(d.db_use_time[0] ne 0. and d.da_use_time[0] ne 0., kboth)
  qnone = where(d.db_use_time[0] eq 0. and d.da_use_time[0] eq 0., knone)
  text_ba = strarr(nd)
  if kb gt 0 then text_ba[qb] = 'Background is from Day Before.'
  if ka gt 0 then text_ba[qa] = 'Background is from Day After.'
  if kboth gt 0 then text_ba[qboth] = 'Background is from average of Day Before and Day After.'
  if knone gt 0 then text_ba[qnone] = 'No background from day before/after available, use in-orbit background.'

  pre = ['List of Flare Data (FD) files with usable background. Generated 23-Apr-2019', $
    '', $
    'For each flare time interval listed, a corresponding background has been computed either', $
    'from day before, day after, or the average of the two and stored in a save file ', $
    '(except for two flares where neither was adequate, but in-orbit background can be used).', $
    'For all other FD files in these directories, no adequate background was provided. In-orbit ', $
    'background subtraction needs to be done.', $
    '']

  out = usename + atd[0,*] + atd[1,*] + '   ' + text_ba
  q = where(usename eq 'F9292_FD', kq)
  if kq gt 0 then out[q] = out[q] + '  Background-subtracted data only available to 13:18 UT.'

  prstr, [pre, out], file='GRS_flare_background.txt'
end

