; Name: spex_image__roi_config
;
; Purpose:  ROI_CONFIG method is a widget interface to let user set ROI selection parameters
;	for imagecubes. (like size of panels in panel display, input save file with pre-defined ROIs,
;	color, etc.)
;
; Written, Kim, August 2005
; Modifications:
;
;----------

pro spex_image::roi_config_update_widget, wbase

widget_control, wbase, get_uvalue=state

widget_control, state.w_infile, set_value=state.object->get(/spex_roi_infile)
widget_control, state.w_outfile, set_value=state.object->get(/spex_roi_outfile)

widget_control, state.w_size, set_value=state.object->get(/spex_roi_size)
widget_control, state.w_expand, set_value=state.object->get(/spex_roi_expand)
widget_control, state.w_color, set_droplist_select=state.object->get(/spex_roi_color)

end

;--------------------------------------------------------------------------

pro spex_roi_config_event,event
widget_control,event.top,get_uvalue=state
if obj_valid(state.object) then state.object->roi_config_event,event
return & end

;--------------------------------------------------------------------------

pro spex_image::roi_config_event, event

widget_control, event.id, get_uvalue=uvalue

wbase = event.top
widget_control, wbase, get_uvalue=state

case uvalue of
	'infile': begin
		widget_control, state.w_infile, get_value=val
		self -> set, spex_roi_infile=val
		end
	'inbrowse': begin
		file = ssw_pickfile( exists=exists, filter='*.sav', $
			title='Select Input ROI File (idl save file)')
		case 1 of
			file eq '': msg = 'ERROR: No file selected.'
			exists eq 0: msg = 'ERROR: Selected file not found - ' + file
			else: msg = ''
		endcase
		if msg eq '' then self -> set, spex_roi_infile=file $
			else message, msg, /cont
		end
	'inlist': self -> list_roi
	'outfile': begin
		widget_control, state.w_outfile, get_value=val
		self -> set, spex_roi_outfile=val
		end
	'outbrowse':begin
		file = ssw_pickfile( filter='*.sav', $
			title='Select Output ROI File (idl save file)')
		if file ne '' then self -> set, spex_roi_outfile=file
		end
	'outlist': self -> list_roi, self->get(/spex_roi_outfile)
	'size': self -> set, spex_roi_size=event.value
	'expand': self -> set, spex_roi_expand=event.value
	'color': self -> set, spex_roi_color=event.index
    'exit': widget_control, wbase, /destroy
    else:
endcase

if xalive(wbase) then self -> roi_config_update_widget, wbase

end

;--------------------------------------------------------------------------

pro spex_image::roi_config, group=group, _extra=_extra

get_font, font, big_font=big_font

wbase = widget_base (title='Region Selection Options', /column, group=group, $
	modal=exist(group), /frame, ypad=30, space=10)

tmp = widget_label (wbase, value='Region Selection Options', font=big_font)

w_file1_base = widget_base (wbase, /row, space=10)
;w_file_col1 = widget_base (w_file_base, /column)
;w_file_col2 = widget_base (w_file_base, /column)
;w_file_col3 = widget_base (w_file_base, /column)
;w_file_col4 = widget_base (w_file_base, /column)

tmp = widget_label (w_file1_base, value='Input ROI File: ')
w_infile = widget_text (w_file1_base, /edit, xsize=50, value='', uvalue='infile')
tmp = widget_button (w_file1_base, value='Browse...', uvalue='inbrowse')
tmp = widget_button (w_file1_base, value='Summarize', uvalue='inlist')

w_file2_base = widget_base (wbase, /row, space=10)
tmp = widget_label (w_file2_base, value='Output ROI File: ')
w_outfile = widget_text (w_file2_base, /edit, xsize=50, value='', uvalue='outfile')
tmp = widget_button (w_file2_base, value='Browse...', uvalue='outbrowse')
tmp = widget_button (w_file2_base, value='Summarize', uvalue='outlist')

w_size_base = widget_base (wbase, /row, space=30)
w_size = cw_edroplist (w_size_base, $
					label='Panel Size:', $
					value=50, $
					xsize=5, $
					drop_values = 50 + indgen(8)*50, $
					uvalue='size' )

w_expand = cw_edroplist (w_size_base, $
					label='Expand factor: ', $
					value=3, $
					drop_values=indgen(5)+1, $
					uvalue='expand')

w_color = widget_droplist (w_size_base, title='Color Table: ', $
					value = trim(indgen(40)), $
					uvalue='color' )

tmp = widget_button (wbase, value='Exit', uvalue='exit', /align_center)

widget_control, wbase, /realize

state = {object: self, $
	w_infile: w_infile, $
	w_outfile: w_outfile, $
	w_size: w_size, $
	w_expand: w_expand, $
	w_color: w_color}

widget_control, wbase, set_uvalue=state

widget_offset, group, newbase=wbase, xoffset, yoffset
widget_control, wbase, xoffset=xoffset, yoffset=yoffset

self -> roi_config_update_widget, wbase

xmanager, 'spex_roi_config', wbase, no_block=0

end