;+
; Name: spex_message
;
; Project: HESSI
;
; Purpose: Generate MESSAGE exception.  HSI_MESSAGE decides whether to use
;   /CONTINUE on MESSAGE call depending on debug level.
;
; Method:  Some GET and GETDATA object methods establish a CATCH error
;  handler.  HSI_MESSAGE is called by routines further down the processing
;  chain from the GET and GETDATA calls to generate an exception that will
;  cause the program flow to abort and jump to the nearest error handler.
;
;  However, in debugging mode, we don't want to change the program flow.
;  We want to continue and let the error condition cause a crash.
;  When the debug level is not 0, then the error handler
;  is NOT set in the objects (see HSI_INSERT_CATCH) and HSI_MESSAGE
;  calls MESSAGE with /CONTINUE.
;
;  This is not a procedure.  It should be inserted into your code via
;  @spex_message after setting the variable msg to your error message string.
;  The reason for this is that in case there is no catch and continue is
;  not set here (based on debug level), then it will stop, so we want it to
;  stop in the routine itself, not in a routine called spex_message.
;
; Calling sequence:
;  Put your error text in a variable called msg
;  @spex_message
;
; Example:
;   msg = 'No lookup table.  Aborting.'
;   @spex_message
;
; Written: Kim Tolbert 22-Oct-2002
; Modifications:
;-
;---------------------------------------------------------------------------


debug = spex_get_debug()

nlines = n_elements(msg)
if nlines gt 1 then for i=0,nlines-2 do message, msg[i], /cont
message, msg[nlines-1], continue=(debug gt 0)
