;+
; Name: spex_smm_grs_specfile
;
; Purpose: Object class for handling smm grs data FITS files in ospex
;
; Written 16-Mar-2019 Kim Tolbert, based on spex_smm_hxrbs_specfile__define.pro
; Modifications:
;  12-Jun-2019, Kim. Added HEM option (call read_grs_4_ospex with /hem when spex_data_sel is 'hem')
;    Note - widget has no button to select hem - this is kind of a hidden data set for Gerry Share. When
;      he wants to use HEM data, from command line he must first set o->set,spex_data_sel='hem'
;
;-

pro spex_smm_grs_specfile_test

  o =  obj_new( 'spex_smm_grs_specfile' )
  o->set,  spex_spec =  'smm_grs_19800521_2027_2132_FD.fits'
  data =  o->getdata()

end

;------------------------------------------------------------------------------

pro spex_smm_grs_specfile::read_data, file_index=file_index, $
  spectrum,  errors,  livetime,  $
  spex_respinfo, spex_file_time,  spex_ut_edges,  spex_ct_edges,  $
  spex_area, spex_title, spex_detectors,  $
  spex_interval_filter, spex_units, spex_data_name, $
  spex_deconvolved, spex_pseudo_livetime, spex_data_pos, spex_def_eband=spex_def_eband, $
  err_code=err_code, _extra=_extra

  file =  self -> get( /spex_specfile )
  checkvar, file_index, 0
  file = file[file_index]
  data_sel = self -> get(/spex_data_sel)
  hem = strlowcase(data_sel) eq 'hem'

  ; why not use hsi_spectrum_fitsread here?  Ask richard. !!!!!
  read_grs_4_ospex, FILES = file, $
    hem = hem, $
    _EXTRA = _ref_extra, $
    data_str = data_str, $
    ERR_CODE = err_code, $
    ERR_MSG = err_msg

  if err_msg[0] ne '' then begin
    ;if not spex_get_nointeractive() then xmessage,err_msg else print,err_msg
    message, err_msg, /cont
    return
  endif

  spectrum =  data_str.rcounts
  errors =  data_str.ercounts
  livetime =  data_str.ltime

  spex_data_name = hem ? 'SMM_GRS_HEM' : 'SMM_GRS'
  spex_respinfo = data_str.respinfo
  spex_file_time = utime([data_str.start_time, data_str.end_time])
  spex_ut_edges = data_str.ut_edges
  spex_ct_edges = data_str.ct_edges
  spex_area = data_str.area[0]
  spex_title = data_str.title
  spex_detectors = data_str.detused
  spex_units = data_str.units
  spex_interval_filter = data_str.atten_states
  spex_deconvolved = 0
  spex_pseudo_livetime = 0
  spex_data_pos = [0.,0.]

end

;------------------------------------------------------------------------------

function spex_smm_grs_specfile::get_source_pos

  return, [-9999.,-9999.]

end

;------------------------------------------------------------------------------

pro spex_smm_grs_specfile__define

  self = {spex_smm_grs_specfile, $
    INHERITS spex_data_strategy }

END
