;+
; Name: spex_smm_hxrbs_specfile
; 
; Purpose: Object class for handling smm hxrbs data in ospex
; 
; Written 19-Feb-2013 Kim Tolbert
; Modifications:
;
;-

pro spex_smm_hxrbs_specfile_test

  o =  obj_new( 'spex_smm_hxrbs_specfile' )
  o->set,  spex_spec =  'hxrbs_6955.fits'
  data =  o->getdata()

end

;------------------------------------------------------------------------------

pro spex_smm_hxrbs_specfile::read_data, file_index=file_index, $
                       spectrum,  errors,  livetime,  $
                       spex_respinfo, spex_file_time,  spex_ut_edges,  spex_ct_edges,  $
                       spex_area, spex_title, spex_detectors,  $
                       spex_interval_filter, spex_units, spex_data_name, $
                       spex_deconvolved, spex_pseudo_livetime, spex_data_pos, spex_def_eband=spex_def_eband, $
                       err_code=err_code, _extra=_extra

  file =  self -> get( /spex_specfile )
  checkvar, file_index, 0
  file = file[file_index]
  
; why not use hsi_spectrum_fitsread here?  Ask richard. !!!!!
  read_hxrbs_4_ospex, FILES = file, $
                      _EXTRA = _ref_extra, $
                      data_str = data_str, $
                      ERR_CODE = err_code, $
                      ERR_MSG = err_msg

  if err_msg[0] ne '' then begin
      ;if not spex_get_nointeractive() then xmessage,err_msg else print,err_msg
      message, err_msg, /cont
      return
  endif

  spectrum =  data_str.rcounts
  errors =  data_str.ercounts
  livetime =  data_str.ltime

  spex_data_name = 'HXRBS'
  spex_respinfo = data_str.respfile
  spex_file_time = utime([data_str.start_time, data_str.end_time])
  spex_ut_edges = data_str.ut_edges
  spex_ct_edges = data_str.ct_edges
  spex_area = data_str.area[0]
  spex_title = data_str.title
  spex_detectors = data_str.detused
  spex_units = data_str.units
  spex_interval_filter = data_str.atten_states
  spex_deconvolved = 0
  spex_pseudo_livetime = 0
  spex_data_pos = [0.,0.]
  

end

;------------------------------------------------------------------------------

function spex_smm_hxrbs_specfile::get_source_pos

read_hessi_4_ospex_params, self -> get( /spex_specfile ), param, status
if status then begin
	if tag_exist(param,'USED_XYOFFSET') then return, param.used_xyoffset else $
		return, [-9999.,-9999.]
endif

end

;------------------------------------------------------------------------------

pro spex_smm_hxrbs_specfile__define

  self = {spex_smm_hxrbs_specfile, $
          INHERITS spex_data_strategy }

END
