;+
; NAME:
;	Ai_bpow
; PURPOSE:
;	the inital values of apar, the fit parameters,
;       for the broken power law.. 
; CALLING SEQUENCE:
;	apar=Ai_bpow(fy,sfy,e)
; INPUT:
;	fy=Log(Photon Flux), 
;	sfy=Unc. in fy squared,
;	e=log(channel energies), 
; OUTPUT:
;	apar=initial values for the fit parameters
; HISTORY:
;       16-nov-95, from ai_2pl.pro, jmm
;-
FUNCTION Ai_bpow, fy, sfy, e

@function_com
;this is needed for epivot, where a(0) is defined as the flux at epivot,

   nch = N_ELEMENTS(e)
   lfit_2, fy, sfy, e, nch, ax  ;fit fy to a double power law
                                ;ax=[log(k1),-gm1,log(k2),-gm2]
                                ;k1 and k2 are defined at 1 keV, or e=0

   ebr = (ax(2)-ax(0))/(-ax(3)+ax(1)) ;calculate the break energy
   ebr = 10.0^ebr               

   IF(epivot LE ebr) THEN a0 = ax(0)+ax(1)*alog10(epivot) $
     ELSE a0 = ax(2)+ax(3)*alog10(epivot)
   
   a = [10.0^a0, -ax(1), ebr, -ax(3)] ;a=[a0,gm1,ebr,gm2]
   
   RETURN, a
END

