;+
; NAME:
;	Ai_thermal
; PURPOSE:
;	the inital values of a for the single thermal.. 
; CALLING SEQUENCE:
;	a=Ai_thermal(fy,sfy,e,nch)
; INPUT:
;	fy=Log(Photon Flux), 
;	sfy=Unc. in fy squared,
;	e=log(channel energies), 
;	nch=no. of channels
; OUTPUT:
;	a=initial values for a, the fit parameters
; HISTORY:
;       16-nov-95, from ai_1th.pro, jmm
;-
FUNCTION Ai_thermal, fy, sfy, e, nch
   
@function_com
;this is needed for epivot, where a(0) is defined as the flux at epivot,

   IF (nch LT 2) THEN BEGIN	;make sure there are 2 channels
      print, ' not enough channels, nch<2'
      a = [0.0, 0.0]
      GOTO, ret1
   ENDIF ELSE BEGIN             ;select the channel closest to twice ch 1
      e11 = e(0)
      e12 = e11+0.3
      tmpmn = 1.0e20
      FOR i = 1, nch-1 DO BEGIN
         tmp = abs(e(i)-e12)
         IF (tmp LT tmpmn) THEN BEGIN
            tmpmn = tmp
            nh = i
         ENDIF
      ENDFOR
   ENDELSE
; now run the fit through the 1st and nhth channels
; t6=11.6*(enh-e11)/(ln(10)*(fy(0)-fy(nh)+e(0)-e(nh)))
; e47=10.0^(fy(0)+e(0)+log(t6)/2-log(3.63e5)+11.6*e11/(ln(10)*t6))
   c1 = 11.6/alog(10.0)
   c2 = alog10(3.63e5)
   
   enh = 10.0^e(nh)
   e11 = 10.0^e(0)
   temp = fy(0)-fy(nh)+e(0)-e(nh)
   IF (temp LE 0.0) THEN BEGIN
      print, ' T < 0 in ai_thermal'
      a = [1.0, 10.0]
      GOTO, ret1
   ENDIF
   
   t6 = c1*(enh-e11)/temp
   e47 = fy(0)+e(0)+(0.5*alog10(t6))-c2+(c1*e11/t6)
   e47 = 10.0^e47
   
   a = [e47/100.0, t6]          ;default is the emission measure in 10^49
   
   ret1:	RETURN, a
END

