;+
;
; NAME:
;	hessi_fits2drm
;
; PURPOSE:
;   Read a HESSI response matrix from a FITS file and return information of
; 	interest to Spex.
;
;
; CATEGORY:
;       SPECTRAL FITTING, SPEX - HESSI
;
; CALLING SEQUENCE:
;
; CALLS:
; 	fits2rm, fxpar
;
; INPUTS:
;
; OUTPUTS:
;
; INPUT KEYWORDS:
; 	FILE - name of FITS file to read.
; 	SFILE - not used.  Included for Spex comptability.
;
; OUTPUT KEYWORDS:
; 	EDGES_OUT - 2 x n_pha_bins array of detector channel energy bins
; 	EDGES_IN - 2 x n_input_bins array of input energy bins
; 	AREA - detector area
; 	DRM - response matrix
;
; PROCEDURE:
;
; Written 25-May-2001 Paul Bilodeau, RITSS/NASA-GSFC
;-
;-------------------------------------------------------------------------------
PRO hessi_fits2drm, FILE=file, SFILE=sfile, EDGES_OUT=edges_out, $
	EDGES_IN=edges_in, AREA=area, DRM=drm

fits2rm, file[0], RM=drm, EBINS=edges_in, DETBINS=edges_out, EXT_HEADER=hdr

area = fxpar( hdr, 'GEOAREA', COUNT=area_cnt )
area = area_cnt GT 0L ? st2num( area ) : 1.

END