;+
; NAME:   edges_hxrs
;
;
; PURPOSE:  return the enrgy boundaries from the lower eight detector channels from HXRS
;
;
; CATEGORY: SPEX, spectral analysis
;
;
; CALLING SEQUENCE:
;	examples
;
;	edges_hxrs, edges
;	edges_hxrs, edges, width=widths
;
;
; INPUTS:
;	none
;
;
; OUTPUTS:
;	edges - array containing HXRS channel edges
;	widths - optional, array with energy widths of HXRS channels
;
;
; SIDE EFFECTS:
;	none
;
;
; RESTRICTIONS:
;	none
;
;
; PROCEDURE:
;	none
;
;
; CALLS:
;	none
;
;
; WRITTEN:
;	Alan L. Kiplinger, CU-Boulder, May 2000
;
;
; MODIFICATION HISTORY:
; 31-May-2000 Paul BIlodeau eliminated upper two channels, as Spex cannot handle channels
;     with overlapping energy ranges
;-

pro edges_hxrs, edges, width=width

edges=fltarr(2,8)


energylo=[12.6,	 19.,   29.,	44.,   67.2,   100.2,  147.2,  219.5 ] ;,   249.5,   12.6]
energyhi=[19.,	 29.,   44.,    67.2,  100.2,  147.2,  219.5,  500.  ] ;,    500.,    500.]

edges(0,*)=energylo
edges(1,*)=energyhi

width=energyhi-energylo

return
end

