;+
; NAME:
;	Lfit_3
; PURPOSE:
;	Obtains a P.L. with a 2 breaks in it
; CALLING SEQUENCE:
;	Lfit_3,y,s2,x,nch,ax,fx
; INPUT:
;	y=obs data, 
;	s2=unc. squared
;	x=energies, 
;	nch=no. of channels
; OUTPUT:
;	fx=ax(0)+ax(1)*x, e<eb1
;	fx=ax(2)+ax(3)*x, eb1<e<eb2
;	fx=ax(4)+ax(5)*x, e>eb2
; HISTORY:
;	Spring,' 92 JMcT
;-
PRO Lfit_3, y, s2, x, nch, ax, fx
   
   chmn = 1.0e20
   nch1 = nch-1
;fit a single pl. and a double pl
   
   FOR n = 1, nch-4 DO BEGIN	;at least 2 pts below eb1,
      ;but 4 above
      n1 = n+1
      y1 = y(0:n)                 ;below break energy
      x1 = x(0:n)
      s21 = s2(0:n)
      lfit, y1, s21, x1, ax1	;fit a power law
      
      y2 = y(n1:nch1)             ;above break energy
      x2 = x(n1:nch1)
      s22 = s2(n1:nch1)
      nc2 = N_ELEMENTS(y2)
      lfit_2, y2, s22, x2, nc2, ax2, fx2 ;double p.l.
      
      dg1 = -ax2(1)+ax1(1)
      eb1 = (ax2(0)-ax1(0))/dg1   ;the break energy will be restricted to x(n)<ebr<x(n1)
      IF(eb1 LT x(n)) THEN eb1 = x(n) ELSE $ ;reset ebr
        IF(eb1 GT x(n1)) THEN eb1 = x(n1) 
      
      ax2o = ax2(0)
      ax2(0) = ax1(0)+dg1*eb1     ;reset k2
      ax2(2) = ax2(2)+(ax2(0)-ax2o) ;reset k3, then
      
      ys1 = ax1(0)+ax1(1)*x1      ;values of y from fit
      ys2 = fx2
      
      ys = [ys1, ys2]		;get chi^2
      ch2 = total((ys-y)^2/s2)
      IF (ch2 LT chmn) THEN BEGIN
         chmn = ch2
         ax = [ax1, ax2]           ;[ax1(0),ax1(1),ax2(0),ax2(1),ax2(2),ax2(3)]
         ;[k1,-gm1,k2,-gm2,k3,-gm3]
         fx = ys
      ENDIF
   ENDFOR
   
   RETURN
END

