
;+
; NAME: spex_current
;	
; PURPOSE: return current value of SPEX variable from common or parameters
;	
; CALLING SEQUENCE: value = spex_current( spex_name, incommon=[0,1])
;	
; INPUTS:
;	spex_name - string with name of variable in common or option list 
; OPTIONAL INPUTS:
;	/incommon - this variable is in the spex_commons.pro common blocks
;	input       - if defined, put this value into the variable, return 1 for success
;	/by_tag   - if set, then convert the tag name values into SPEX variables
;		    used on input only
;	/SAVE      - if set, then argument is interpreted as a common separated list of
;		    variables to be written to spex.sav or the given
;		    FILENAME
;	/RESTORE   - if set then restore spex.sav or the given filename.
; OUTPUTS:
;	returns the value of the variable for output or 1/0 for success/fail on input
;
; OPTIONAL OUTPUTS:
;
; PROCEDURE:
;
; CALLS:
;
; COMMON BLOCKS:
;	those in spex_commons.pro and dep150_com 
;
; RESTRICTIONS:
;	should not be used where round-off error from string conversion will be a problem
;
; MODIFICATION HISTORY:
;	
;
;
; 
;
; value = spex_current( spex_name, incommon=[0,1])
;
; Avoids including common block in every procedure which needs to access
; main level spex variables.  Also extracts variable values from the fields
; in prm_spex, the option structures.
; 
; Mod:
;	ras, jul-94
;	ras, mar-95, now it won't crash if you forget /incommon for variables
;		in the common block
;	ras, 30-mar-95, added input capability    
;	ras, 12-oct-95, added by_tag
;	ras, 4-april-1996, added save and restore and filename
;-
;just to get PARAMETERS and PRM_SPEX

function spex_current, prog_name, incommon=incommon, input=input, $
	by_tag=by_tag,  error=error, save=save, restore=restore, $
	filename=filename

error = 1

@spex_commons

common dep150_com, field, spex_name, value, value_out, its_common, its_direct

if keyword_set(save) then begin
	checkvar, filename, 'spex.sav'
	result = execute('save,file='+filename+','+prog_name)
	return, 1
end
if keyword_set(restore) then begin
	checkvar, filename, 'spex.sav'
	filename = findfile(filename, count=fcount)
	if fcount eq 0 then return,0
	result = execute('restore,file='+filename(0)+','+prog_name)
	return,1
end
	
if keyword_set(by_tag) and datatype(input) eq 'STC' then begin
	prog_names = tag_names(input)
	inputs     = input
endif

checkvar, prog_names, prog_name

for i=0,n_elements(prog_names)-1 do begin

  prog_name = prog_names(i)
  if datatype(inputs) eq 'STC' then input = inputs.(i)

  its_common = fcheck( incommon,0)
  spex_name = fcheck( prog_name, '')

  ;What if SPEX ain't defined yet, return with an error code
  if n_elements(parameters) eq 0 then return, 0

  tag = (where(strupcase(spex_name) eq parameters))(0)

  if n_elements(input) eq 0 then begin
  if not its_common and tag ne -1 then begin
   value = 0
   nv  = prm_spex(tag).numval
   if nv gt 1 then temp = fltarr(nv) else temp=0.
   r = execute( 'value = ' + prm_spex(tag).typ + '(temp)')
   reads, prm_spex(tag).val, value 
  endif  else $
   r= execute('value = fcheck( '+spex_name+', 0) ')

  endif else begin  ;load the variable with the input value
    value = 1
    if not its_common and tag ne -1 then prm_spex(tag).val = input 
    value = execute(spex_name+'=input')
  endelse

endfor

error = 0
return, value
end
