;+
; NAME:
;	SPEX_FCHISQR
; PURPOSE:
;	Compute statistic of total Chisqr for a function (FUNCTION_NAME),
;	a set of parameters A, independent variables X, dependent variables Y,
;	and weighting factors, W.  Also evaluates the function at X for A as
;	well as the partial derivatives wrt the parameters if requested.
; CATEGORY:
;	E2 - Curve and Surface Fitting
; CALLING SEQUENCE:
;	CHISQR = SPEX_FCHISQR(X,Y,W,Function_Name,A,YFIT,PDER) 
; INPUTS:
;	X = Row vector of independent variables.
;	Y = Row vector of dependent variable, same length as x.
;	W = Row vector of weights, same length as x and y.
;		For no weighting
;		w(i) = 1., instrumental weighting w(i) =
;		1./y(i), etc.
;	Function_name - Name of function (string) with X and A as arguments.
;	A = Vector of nterms length of parameters for each parameter.  
;	If A is double precision, calculations are performed in double 
;	precision, otherwise in single prec.
;
; Optional:
;	free - vector of indices of free parameters for which derivs are needed
; OUTPUTS:
;;	Function result = YFIT = Vector of calculated
;		values.
; OPTIONAL OUTPUT PARAMETERS:
;	Partial Derivatives = PDER
;	
; COMMON BLOCKS:
;	NONE.
; SIDE EFFECTS:
;	See details in CURVEFIT.
; RESTRICTIONS:
;	NONE.
; PROCEDURE:
;	Copied from "CURFIT", least squares fit to a non-linear
;	function, pages 237-239, Bevington, Data Reduction and Error
;	Analysis for the Physical Sciences.
;-
 FUNCTION SPEX_FCHISQR,X,Y,W,Function_Name,A,YFIT,PDER, free=free

;	ON_ERROR,2		;RETURN TO CALLER IF ERROR
	IF N_PARAMS(0) EQ 7 THEN $
		FUNCT1,Function_name,X,A, YFIT,PDER, free=free ELSE $	;EVALUATE FUNCTION
		FUNCT1,Function_name,X,A,YFIT	;EVALUATE FUNCTION
        	CHISQR = TOTAL(W*(Y-YFIT)^2) ;TOTAL CHISQR
	RETURN,CHISQR		;RETURN RESULT
END
