
;+
; NAME: wsetshow
;
; PURPOSE: Make the desired window active when switching from
;	   spectral and temporal plotting domains.  Also,
;	   loads in the correct sysvariables for those windows.
;	   Used in SPEX for flipping between windows
; CALLING SEQUENCE: wsetshow, i
;
; INPUTS: iout - window index to bring up, novalue if just putting plotting
;	      values in common
; INPUT KEYWORDS:
;         /noshow - set the window, but don't raise it
;         time_win_id - set during initialization of SPEX.  Window
;                       number of time history window.
;         spec_win_id - set during initialization of SPEX.  Window
;                       number of spectral window.
;         /initialize - set during initialization of SPEX.  Set
;                       time_id and spec_id to values to corresponding
;                       win keywords.  Exit after setting these
;                       values.
;         /timehistory, /spectral - sets iout to number of appropriate window.
; MODIFICATION HISTORY:
;	ras, 22-apr-94, include sysvariables
;	ras, 19-may-94, protect against current window not being 0 or 1
;	ras, 8-april-1996, always use old ! variables, even if redundant
;	richard.schwartz@gsfc.nasa.gov, 18-mar-1998, use xdevice() for
;	'X'
;       Paul Bilodeau, 20-March-2002, added time_win_id, spec_win_id,
;       initialize, and win_name keywords.  Added time_id and spec_id
;       parameters to wsetshow_common block.
;       Paul BIlodeau, 17-September-2002, added checks for definition
;       of window id numbers with and without /init, fix crash for
;       running in /cmode with the 'Z' device.  Documentation updates.
;-
;
PRO wsetshow, iout, $
              noshow=noshow, $
              time_win_id=time_win_id, $
              spec_win_id=spec_win_id, $
              initialize=init, $
              TIMEHISTORY=timehistory, $
              SPECTRAL=spectral

COMMON wsetshow, time_id, spec_id, time_stc, spec_stc

IF n_elements(time_stc) eq 0 THEN $
  time_stc ={winsav, x:!x, y:!y, clip:!p.clip}

IF n_elements(spec_stc) eq 0 THEN $
  spec_stc ={winsav, x:!x, y:!y, clip:!p.clip}

COMMON spex_proc_com

IF  Keyword_Set( init ) THEN BEGIN
    have_time_win_id = N_Elements( time_win_id ) GT 0
    have_spec_win_id = N_Elements( spec_win_id ) GT 0
    IF NOT( have_time_win_id AND have_spec_win_id ) THEN BEGIN
        IF spex_debug THEN $
          printx, "WSETSHOW: No window ID's for initialization."
    ENDIF ELSE BEGIN
        time_id = time_win_id
        spec_id = spec_win_id
    ENDELSE
    RETURN
ENDIF ELSE BEGIN
    have_time_id = N_Elements( time_id ) GT 0
    have_spec_id = N_Elements( spec_id ) GT 0
    ;; Just exit.  This can happen when exiting after running in
    ;; /cmode.  This is not necessarily an error.
    IF NOT( have_time_id AND have_spec_id ) THEN RETURN
ENDELSE

checkvar, iout, -1	;if just saving state

IF Keyword_Set( timehistory ) THEN iout = time_id
IF Keyword_Set( spectral ) THEN iout = spec_id

device_in = !d.name

IF device_in NE xdevice() THEN set_plot, xdevice()

in = !d.window	;active window

IF iout NE in OR fcheck(test,0) NE 0 THEN BEGIN
    CASE in OF
        -1: BEGIN
            printx, 'Call to WSETSHOW with no active window!'
            winold = time_stc
        END
        time_id: begin
            time_stc ={winsav, x:!x, y:!y, clip:!p.clip}
            winold = spec_stc
        END
        spec_id: BEGIN
            spec_stc ={WINSAV, X:!X, Y:!Y, CLIP:!P.CLIP}
            winold=time_stc
        END
        ELSE: BEGIN
            printx,'WARNING! WSETSHOW USE IS NON-STANDARD. ' + $
                   'NEXT OVERPLOT MAY BE WRONG!'
            winold = iout EQ spec_id ? spec_stc : time_stc
        ENDELSE
    ENDCASE

	; AKT - CHECK IF RANGE HAS BEEN SET FIRST (A PLOT HAS BEEN DONE)
        IF Total(winold.X.crange) NE 0.0 AND iout NE -1 THEN BEGIN
            ;; SET THE AXES
            reset_xy, winold
        ENDIF
    ENDIF

IF iout NE -1 THEN BEGIN
    wset, iout
    IF NOT fcheck(nshow, 0) THEN WShow, iout
ENDIF

set_plot, device_in

END
